<?php
/**
* 2007-2018 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2018 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

class Transactionale extends Module implements WidgetInterface {

    public function __construct() {
        $this->name = 'transactionale';
        $this->tab = 'advertising_marketing';
        $this->version = '2.0.0';
        $this->author = 'Transactionale';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->getTranslator()->trans('Transactionale', array(), 'Module.Transactionale.Admin');
        $this->description = $this->getTranslator()->trans('Transactionale integration for Advertisers and Publishers', array(), 'Module.Transactionale.Admin');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => '1.7.99.99');
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install() {
        Configuration::updateValue('TR_AUTH_KEY', '');
        Configuration::updateValue('TR_COUNTRY', '');
        Configuration::updateValue('TR_TOUCHPOINT_TYPE', 0);
        Configuration::updateValue('TR_WEB_TOUCHPOINT_ID', '');
        Configuration::updateValue('TR_WEB_TOUCHPOINT_TEXT', '');
        Configuration::updateValue('TR_IMPORT_LEADS_AUTO', false);

        return parent::install() && $this->registerHook('header') && $this->registerHook('displayOrderConfirmation') && $this->registerHook('actionValidateOrder');
    }

    public function uninstall() {
        Configuration::deleteByName('TR_AUTH_KEY');
        Configuration::deleteByName('TR_COUNTRY');
        Configuration::deleteByName('TR_TOUCHPOINT_TYPE');
        Configuration::deleteByName('TR_WEB_TOUCHPOINT_ID');
        Configuration::deleteByName('TR_WEB_TOUCHPOINT_TEXT');
        Configuration::deleteByName('TR_IMPORT_LEADS_AUTO');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent() {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitTransactionaleSettings'))) {
          
          $authKey = Tools::getValue('TR_AUTH_KEY');
          $country = Tools::getValue('TR_COUNTRY');
          $errors = array();
          
          if (empty($authKey)) {
            $errors[] = $this->getTranslator()->trans('Auth Key is required', array(), 'Module.Transactionale.Admin');
          }
          if (empty($country)) {
            $errors[] = $this->getTranslator()->trans('Country is required', array(), 'Module.Transactionale.Admin');
          } else if (Tools::strlen($country) != 2) {
            $errors[] = $this->getTranslator()->trans('Country is invalid', array(), 'Module.Transactionale.Admin');
          }
          
          if (count($errors)) {
            $this->context->smarty->assign('errors', $errors);
          } else {
            $this->postProcess();
          }
        } else if (((bool)Tools::isSubmit('submitOptIn'))) {
            Configuration::updateValue('PS_CUSTOMER_OPTIN', 1);
        }

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm() {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitTransactionaleSettings';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm() {
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->getTranslator()->trans('Settings', array(), 'Module.Transactionale.Admin'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'TR_AUTH_KEY',
                        'required' => true,
                        'label' => $this->getTranslator()->trans('Auth Key', array(), 'Module.Transactionale.Admin'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'TR_COUNTRY',
                        'required' => true,
                        'label' => $this->getTranslator()->trans('Country', array(), 'Module.Transactionale.Admin'),
                        'desc' => 'Your Country ISO "Alpha-2" Code.',
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->getTranslator()->trans('Touchpoint type', array(), 'Module.Transactionale.Admin'),
                        'name' => 'TR_TOUCHPOINT_TYPE',
                        'options' => array(
                          'query' => array(
                            array(
                              'id' => '0',
                              'name' => $this->getTranslator()->trans('Mail', array(), 'Module.Transactionale.Admin'),
                            ),
                            array(
                              'id' => '1',
                              'name' => $this->getTranslator()->trans('Web', array(), 'Module.Transactionale.Admin'),
                            ),
                            array(
                              'id' => '2',
                              'name' => $this->getTranslator()->trans('Mail + Web', array(), 'Module.Transactionale.Admin'),
                            ),
                          ),
                          'id' => 'id',
                          'name' => 'name'
                        )
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Web touchpoint ID', array(), 'Module.Transactionale.Admin'),
                        'name' => 'TR_WEB_TOUCHPOINT_ID',
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Web touchpoint text', array(), 'Module.Transactionale.Admin'),
                        'name' => 'TR_WEB_TOUCHPOINT_TEXT',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->getTranslator()->trans('Import leads automatically', array(), 'Module.Transactionale.Admin'),
                        'desc' => $this->getTranslator()->trans('Paste the above URL in your Transactionale account in the My Account -> Integration section.', array(), 'Module.Transactionale.Admin'),
                        'name' => 'TR_IMPORT_LEADS_AUTO',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->getTranslator()->trans('Enabled', array(), 'Module.Transactionale.Admin')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->getTranslator()->trans('Disabled', array(), 'Module.Transactionale.Admin')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->getTranslator()->trans('Save', array(), 'Module.Transactionale.Admin')
                )
            )
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues() {
        return array(
            'TR_AUTH_KEY'            => Configuration::get('TR_AUTH_KEY'),
            'TR_COUNTRY'             => Configuration::get('TR_COUNTRY'),
            'TR_TOUCHPOINT_TYPE'     => Configuration::get('TR_TOUCHPOINT_TYPE'),
            'TR_WEB_TOUCHPOINT_ID'   => Configuration::get('TR_WEB_TOUCHPOINT_ID'),
            'TR_WEB_TOUCHPOINT_TEXT' => Configuration::get('TR_WEB_TOUCHPOINT_TEXT'),
            'TR_IMPORT_LEADS_AUTO'   => Configuration::get('TR_IMPORT_LEADS_AUTO')
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess() {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader() {
      if (Configuration::get('TR_AUTH_KEY') == '' || Configuration::get('TR_COUNTRY') == '')
        return;
      
      $controller_name = $this->context->controller->php_self;
      $other_controller = Configuration::get('TR_ORDER_CONVERSION'); //for other order-confirmation tpl's
      
      if ($controller_name == 'product') {
        $product = new Product(Tools::getValue('id_product'), false, $this->context->language->id);
        $infoCategory = Category::getCategoryInformations(array($product->id_category_default));
        
        $this->smarty->assign(array(
          'itemId'                  => $product->id,
          'itemPrice'               => $product->getPrice(true, null, 2),
          'itemName'                => $product->name,
          'itemCategoryDefaultId'   => $product->id_category_default,
          'itemCategoryDefaultName' => $infoCategory[$product->id_category_default]['name'],
          'itemEan'                 => $product->ean13
          ));
      } else if ($controller_name == 'category') {
        $category = new Category(Tools::getValue('id_category'), $this->context->language->id);
        
        $count = $category->getProducts($this->context->language->id, 1, 1, null, null, true);
        $per_page = Configuration::get('PS_PRODUCTS_PER_PAGE');
        $res = $count % $per_page ? 1 : 0;
        $pages = floor(($count / $per_page)) + $res;        
        $page = (int)Tools::getValue('p', 1);
        if ($page > $pages || $page < 1)
          $page = 1;
        
        $products = $category->getProducts($this->context->language->id, $page, $per_page);
        $cat_product = array();
        foreach ($products as $product) {
          $obj_product = new Product($product['id_product'], false, $this->context->language->id);
          $infoCategory = Category::getCategoryInformations(array($obj_product->id_category_default));
          
          $cat_product[$obj_product->id]['id'] = $obj_product->id;
          $cat_product[$obj_product->id]['price'] = $obj_product->getPrice(true, null, 2);
          $cat_product[$obj_product->id]['name'] = $obj_product->name;
          $cat_product[$obj_product->id]['category_default_id'] = $obj_product->id_category_default;
          $cat_product[$obj_product->id]['category_default_name'] = $infoCategory[$obj_product->id_category_default]['name'];
        }
        $products = $cat_product;
        $this->smarty->assign(array(
          'products' => $products
          ));
      } else if ($controller_name == 'cart') {
        $cart = new Cart($this->context->cart->id);
        $products = $cart->getProducts();
        $cart_product = array();
        
        foreach ($products as $product) {
          $obj_product = new Product($product['id_product'], false, $this->context->language->id);
          $infoCategory = Category::getCategoryInformations(array($obj_product->id_category_default));
          
          $cart_product[$obj_product->id]['id'] = $obj_product->id;
          $cart_product[$obj_product->id]['quantity'] = $product['quantity'];
          $cart_product[$obj_product->id]['price'] = $obj_product->getPrice(true, null, 2);
          $cart_product[$obj_product->id]['name'] = $obj_product->name;
          $cart_product[$obj_product->id]['category_default_id'] = $obj_product->id_category_default;
          $cart_product[$obj_product->id]['category_default_name'] = $infoCategory[$obj_product->id_category_default]['name'];
        }
        $products = $cart_product;
        $this->smarty->assign(array(
          'products' => $products
          ));
      } else if ($controller_name == 'order-confirmation' || (!empty($other_controller) && $other_controller != '')) {
        
        $id = '';
        if (Tools::getIsset('id_order')) {
          $id = Tools::getValue('id_order');
        } else {
          if (!empty($other_controller) && $other_controller != '') {
            $ids = explode(":", $other_controller);
            if ((int)$this->context->cookie->id_customer == (int)$ids[1]) {
              $id = $ids[0];
            }
          }
        }
        Configuration::deleteByName('TR_ORDER_CONVERSION');
        
        $order = new Order($id);
        if (!empty($id) || Validate::isLoadedObject($order)) {
          $coupon = array();
          foreach ($order->getCartRules() as $cart_rule) {
            $coupon[] = $cart_rule['name'];
          }
          $currency = Currency::getCurrency($order->id_currency);
          $this->smarty->assign(array(
            'orderId'       => $order->id,
            'orderSubtotal' => Tools::ps_round($order->total_products_wt + $order->total_wrapping, Configuration::get('PS_PRICE_DISPLAY_PRECISION')),
            'orderShipping' => Tools::ps_round($order->total_shipping, Configuration::get('PS_PRICE_DISPLAY_PRECISION')),
            'orderDiscount' => Tools::ps_round($order->total_discounts, Configuration::get('PS_PRICE_DISPLAY_PRECISION')),
            'orderTotal'    => Tools::ps_round($order->total_paid, Configuration::get('PS_PRICE_DISPLAY_PRECISION')),
            'orderCoupon'   => implode(", ", $coupon),
            'orderCurrency' => $currency['iso_code']
            ));

          $cart = new Cart($order->id_cart);
          $products = $cart->getProducts();

          foreach ($products as $product) {
            $obj_product = new Product($product['id_product'], false, $this->context->language->id);
            $infoCategory = Category::getCategoryInformations(array($obj_product->id_category_default));

            $cart_product[$obj_product->id]['id'] = $obj_product->id;
            $cart_product[$obj_product->id]['price'] = $obj_product->getPrice(true, null, 2);
            $cart_product[$obj_product->id]['quantity'] = $product['quantity'];
            $cart_product[$obj_product->id]['category_default_name'] = $infoCategory[$obj_product->id_category_default]['name'];
            $cart_product[$obj_product->id]['ean'] = $product['ean13'];
          }
          $products = $cart_product;
          $this->smarty->assign(array(
            'products' => $products
            ));

          $address = new Address($order->id_address_delivery);
          $country = Country::getIsoById($address->id_country);
          $this->smarty->assign(array(
            'address'            => $address->address1,
            'address2'           => $address->address2,
            'addressCity'        => $address->city,
            'addressPostalCode'  => $address->postcode,
            'addressCountry'     => $country,
            'addressPhone'       => $address->phone,
            'addressPhoneMobile' => $address->phone_mobile
            ));

          $customer = new Customer($order->id_customer);
          $this->smarty->assign(array(
            'customerFirstName' => $customer->firstname,
            'customerLastName'  => $customer->lastname,
            'customerCompany'   => $address->company,
            'customerEmail'     => $customer->email,
            'customerBirthDate' => $customer->birthday,
            'customerGender'    => $customer->id_gender == 1 ? 'm' : 'f',
            'customerOptIn'     => $customer->optin ? 'true' : 'false'
            ));

          if (Configuration::get('TR_TOUCHPOINT_TYPE') == 2)
            $touchPointType = 'auto';
          elseif (Configuration::get('TR_TOUCHPOINT_TYPE') == 1)
            $touchPointType = 'web';
          else
            $touchPointType = 'mail';

          $this->smarty->assign(array(
            'touchPointType' => $touchPointType,
            'touchPointId'   => Configuration::get('TR_WEB_TOUCHPOINT_ID')
            ));
        }
      }
      
      $this->smarty->assign(array(
          'tr_debug'   => _PS_MODE_DEV_,
          'tr_authKey' => Configuration::get('TR_AUTH_KEY'),
          'tr_country' => Configuration::get('TR_COUNTRY'),
          'tr_ps'      => _PS_VERSION_,
          'tr_module'  => $this->version
          ));

      return $this->display(__FILE__, '/views/templates/hook/header.tpl');
    }
    
    public function hookactionValidateOrder($params) {
      $id = '';
      if (isset($params['objOrder'])) {
        $order = $params['objOrder'];
      } elseif (isset($params['order'])) {
        $order = $params['order'];
      }
      if (isset($order->id)) {
        $id = $order->id;

        if ($id == '') {
          $id = $order->id_order;
        }

        $orderstring = $id.':'.$order->id_customer;
        Configuration::updateValue('TR_ORDER_CONVERSION', $orderstring);
      }
    }
    
    public function renderWidget($hookName, array $configuration) {
      if ($hookName == 'displayOrderConfirmation') {
        $this->context->smarty->assign($this->getWidgetVariables($hookName, $configuration));
        
        return $this->fetch('module:'.$this->name.'/views/templates/hook/order-confirmation.tpl');
      }
    }
    
    public function getWidgetVariables($hookName, array $configuration) {
      if ($hookName == 'displayOrderConfirmation') {
        if (Configuration::get('TR_TOUCHPOINT_TYPE') != 0) {
        return [
            'web_touchpoint_text' => Configuration::get('TR_WEB_TOUCHPOINT_TEXT'),
            'optin' => Configuration::get('TR_TOUCHPOINT_TYPE') == 2 ? Context::getContext()->customer->optin : 0
          ];
        }
      }
    }
}
